@@ -1,22 +1,18 @@
 package Net::DNS::RR::SVCB;
 
-#
-# $Id: SVCB.pm 1798 2020-07-31 08:39:25Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1798 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: SVCB.pm 1832 2021-03-22 08:33:36Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::SVCB - DNS SVCB resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
@@ -35,7 +31,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	$self->{SvcPriority} = unpack( 'n', $rdata );
 
 	my $index;
-	( $self->{TargetName}, $index ) = decode Net::DNS::DomainName( \$rdata, 2 );
+	( $self->{TargetName}, $index ) = Net::DNS::DomainName->decode( \$rdata, 2 );
 
 	my $params = $self->{SvcParams} = {};
 	my $limit  = length($rdata) - 4;
@@ -46,47 +42,48 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	}
 	die 'corrupt RDATA in ' . $self->type unless $index == $self->{rdlength};
 	$self->_post_parse;
+	return;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	my ($params) = grep defined, $self->{SvcParams}, {};
+	my ($params) = grep {defined} $self->{SvcParams}, {};
 	my @params;
 	foreach ( sort { $a <=> $b } keys %$params ) {
 		my $value = $params->{$_};
 		next unless defined $value;
 		push @params, pack( 'n2a*', $_, length($value), $value );
 	}
-	pack 'n a* a*', $self->{SvcPriority}, $self->{TargetName}->encode, join '', @params;
+	return pack 'n a* a*', $self->{SvcPriority}, $self->{TargetName}->encode, join '', @params;
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my ($params) = grep defined, $self->{SvcParams}, {};
-	my @keys = keys %$params;
-	return ( $self->{SvcPriority}, $self->{TargetName}->string ) unless scalar @keys;
+	my ($params) = grep {defined} $self->{SvcParams}, {};
+	my $priority = $self->{SvcPriority};
+	return ( $priority, $self->{TargetName}->string ) unless scalar keys %$params;
 
-	my @rdata  = unpack 'H4', pack 'n', $self->{SvcPriority};
+	my @rdata  = unpack 'H4', pack 'n', $priority;
 	my $target = $self->{TargetName}->encode();
 	my $length = 2 + length $target;
 	my @target = split /(\S{32})/, unpack 'H*', $target;
-	$target[$#target] .= join ' ', "\t;", $self->{TargetName}->string if $length > 3;
-	push @rdata, $length > 18 ? "\n" : (), @target, "\n";
+	push @rdata, $length > 18 ? "\t; $priority\n" : (), @target;
+	push @rdata, join '', "\t; ", $self->{TargetName}->string, "\n" if $length > 3;
 
-	foreach ( sort { $a <=> $b } @keys ) {
+	foreach ( sort { $a <=> $b } keys %$params ) {
 		my $value = $params->{$_};
 		next unless defined $value;
+		push @rdata, "\n";
 		push @rdata, "; key$_=...\n" if $_ > 15;
 		push @rdata, unpack 'H4H4', pack( 'n2', $_, length $value );
 		push @rdata, split /(\S{32})/, unpack 'H*', $value;
-		push @rdata, "\n";
 		$length += 4 + length $value;
 	}
-	return ( '\\#', $length, @rdata );
+	return ( "\\# $length", @rdata );
 }
 
 
@@ -112,6 +109,7 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 			$self->$1(@argument);
 		}
 	}
+	return;
 }
 
 
@@ -120,10 +118,13 @@ sub _post_parse {			## parser post processing
 
 	my $params = $self->{SvcParams} || return;
 	my %unique;
-	my @unique = grep !$unique{$_}++, unpack 'n*', $params->{0} || return;
+	my @unique = grep { !$unique{$_}++ } unpack 'n*', ( $params->{0} || return );
 	croak( $self->type . qq(: mandatory "key0" not permitted) ) if $unique{0};
-	map croak( $self->type . qq(: duplicate "key$_" in mandatory list) ), grep --$unique{$_}, @unique;
-	map croak( $self->type . qq(: mandatory "key$_" not defined) ), grep !defined( $params->{$_} ), @unique;
+	foreach (@unique) {
+		croak( $self->type . qq(: duplicate "key$_" in mandatory list) ) if --$unique{$_};
+		croak( $self->type . qq(: mandatory "key$_" not defined) )	 if !defined( $params->{$_} );
+	}
+	return;
 }
 
 
@@ -131,6 +132,7 @@ sub _defaults {				## specify RR attribute default values
 	my $self = shift;
 
 	$self->_parse_rdata(qw(0 .));
+	return;
 }
 
 
@@ -138,14 +140,14 @@ sub svcpriority {
 	my $self = shift;					# uncoverable pod
 
 	$self->{SvcPriority} = 0 + shift if scalar @_;
-	$self->{SvcPriority} || 0;
+	return $self->{SvcPriority} || 0;
 }
 
 
 sub targetname {
 	my $self = shift;					# uncoverable pod
 
-	$self->{TargetName} = new Net::DNS::DomainName(shift) if scalar @_;
+	$self->{TargetName} = Net::DNS::DomainName->new(shift) if scalar @_;
 
 	my $target = $self->{TargetName} ? $self->{TargetName}->name : return;
 	return $target unless $self->{SvcPriority};
@@ -160,7 +162,7 @@ use constant ASCII => chr(65) eq 'A';
 my %escape = do {			## precalculated ASCII escape table
 	my @escape = ( 0 .. 32, 34, 92, 127 .. 255 );		# numerical escape
 
-	my %table = map( ( $_ => $_ ), map pack( 'C', $_ ), ( 0 .. 255 ) );
+	my %table = map { ( chr($_) => chr($_) ) } ( 0 .. 255 );
 
 	foreach my $codepoint (@escape) {
 		my $ddd = sprintf( '%03u', $codepoint );
@@ -171,38 +173,36 @@ my %escape = do {			## precalculated ASCII escape table
 	%table;
 };
 
-
 sub _raw {				## concatenate @_ and escape non-printable
-	return @_ unless scalar @_;
-	for ( join '', map $escape{$_}, map split(//), @_ ) {
+	return () unless scalar @_;
+	local $_ = join '', map { $escape{$_} } map { split( //, $_ ) } @_;
 
-		# partial transliteration for non-ASCII character encodings
-		tr
-		[\040-\176\000-\377]
-		[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
+	# partial transliteration for non-ASCII character encodings
+	tr
+	[\040-\176\000-\377]
+	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
 
-		return $_;
-	}
+	return $_;
 }
 
 sub _base64 {
-	_raw( map MIME::Base64::decode($_), @_ );
+	return _raw( map { MIME::Base64::decode($_) } @_ );
 }
 
 sub _integer16 {
-	_raw( map pack( 'n', $_ ), @_ );
+	return _raw( map { pack( 'n', $_ ) } @_ );
 }
 
 sub _ipv4 {
-	_raw( map bless( {}, 'Net::DNS::RR::A' )->address($_), @_ );
+	return _raw( map { bless( {}, 'Net::DNS::RR::A' )->address($_) } @_ );
 }
 
 sub _ipv6 {
-	_raw( map bless( {}, 'Net::DNS::RR::AAAA' )->address($_), @_ );
+	return _raw( map { bless( {}, 'Net::DNS::RR::AAAA' )->address($_) } @_ );
 }
 
 sub _string {
-	_raw( map Net::DNS::Text->new($_)->encode(), @_ );
+	return _raw( map { Net::DNS::Text->new($_)->encode() } @_ );
 }
 
 
@@ -212,44 +212,46 @@ my %keybyname = (
 	'no-default-alpn' => 2,
 	port		  => 3,
 	ipv4hint	  => 4,
-	echconfig	  => 5,
+	ech		  => 5,
+	echconfig	  => 5,					# draft-ietf-dnsop-svcb-https compatible
 	ipv6hint	  => 6,
 	);
 
 
-sub mandatory {				## mandatory=1,2,...
-	my $self = shift;
-	my @keys = grep defined, map m/^key(\d+)/i ? $1 : $keybyname{lc $_}, @_;
-	$self->key0( _integer16( sort { $a <=> $b } @keys ) );
+sub mandatory {				## mandatory=key1,port,...
+	my ( $self, @arg ) = grep {defined} @_;
+	my @keys = map { $keybyname{lc $_} || ( /^key(\d+)$/i ? $1 : croak qq["$_" unknown] ) } @arg;
+	return $self->key0( _integer16( sort { $a <=> $b } @keys ) );
 }
 
 sub alpn {				## alpn=h3,h2,...
 	my $self = shift;
-	$self->key1( _string(@_) );
+	return $self->key1( _string(@_) );
 }
 
 sub no_default_alpn {			## no-default-alpn
-	shift->key2( map '', @_ );				# uncoverable pod
+	return shift->key2( map {''} @_ );			# uncoverable pod
 }
 
 sub port {				## port=1234
 	my $self = shift;
-	$self->key3( map _integer16($_), @_ );
+	return $self->key3( map { _integer16($_) } @_ );
 }
 
 sub ipv4hint {				## ipv4hint=192.0.2.1,...
 	my $self = shift;
-	$self->key4( _ipv4(@_) );
+	return $self->key4( _ipv4(@_) );
 }
 
-sub echconfig {				## echconfig=base64string
+sub ech {				## ech=base64string
 	my $self = shift;
-	$self->key5( map _base64($_), @_ );
+	return $self->key5( map { _base64($_) } @_ );
 }
+sub echconfig { return &ech; }					# uncoverable pod
 
 sub ipv6hint {				## ipv6hint=2001:DB8::1,...
 	my $self = shift;
-	$self->key6( _ipv6(@_) );
+	return $self->key6( _ipv6(@_) );
 }
 
 
@@ -257,13 +259,14 @@ our $AUTOLOAD;
 
 sub AUTOLOAD {				## Dynamic constructor/accessor methods
 	my $self = shift;
+
 	my ($method) = reverse split /::/, $AUTOLOAD;
 
 	my $default = join '::', 'SUPER', $method;
 	return $self->$default(@_) unless $method =~ /^key(\d+)$/i;
 	my $key = $1;
 
-	my ($params) = grep defined, $self->{SvcParams}, {};
+	my ($params) = grep {defined} $self->{SvcParams}, {};
 
 	if ( scalar @_ ) {
 		my $arg = shift;				# keyNN($value);
@@ -274,7 +277,7 @@ sub AUTOLOAD {				## Dynamic constructor/accessor methods
 	}
 
 	my $value = $params->{$key};
-	return defined($value) ? _raw($value) : $value if defined wantarray;
+	return defined($value) ? _raw($value) : $value;
 }
 
 
@@ -285,7 +288,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name HTTPS SvcPriority TargetName alpn=h3,...');
+    $rr = Net::DNS::RR->new('name HTTPS SvcPriority TargetName alpn=h3,...');
 
 =head1 DESCRIPTION
 
@@ -327,23 +330,23 @@ service is not available or does not exist.
 For ServiceMode SVCB RRs, a TargetName of "." indicates that the
 owner name of this record must be used as the effective TargetName.
 
-=head2 mandatory, alpn, no-default-alpn, port, ipv4hint, echconfig, ipv6hint
+=head2 mandatory, alpn, no-default-alpn, port, ipv4hint, ech, ipv6hint
 
-    $rr = new Net::DNS::RR( 'svc.example. SVCB 1 svc.example. port=1234' );
+    $rr = Net::DNS::RR->new( 'svc.example. SVCB 1 svc.example. port=1234' );
 
     $rr->port(1234);
     $string = $rr->port();	# \004\210
     $rr->key3($string);
 
-Constructor methods for mnemonic SvcParams defined in draft-ietf-dnsop-svcb-https-01.
+Constructor methods for mnemonic SvcParams defined in draft-ietf-dnsop-svcb-https-02.
 When invoked without arguments, the methods return the presentation format
 value for the underlying key.
 The behaviour with undefined arguments is not specified.
 
 =head2 keyNN
 
-    $keyNN = $rr->keyNN;
-    $rr->keyNN( $keyNN );
+    $keynn = $rr->keyNN;
+    $rr->keyNN( $keynn );
 
 Generic constructor and accessor methods for SvcParams.
 The key index NN is a decimal integer in the range 0 .. 65534.
@@ -382,6 +385,6 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, draft-ietf-dnsop-svcb-https-01
+L<perl>, L<Net::DNS>, L<Net::DNS::RR>, draft-ietf-dnsop-svcb-https-02
 
 =cut
